@@ -22,8 +22,9 @@ import (
 
 var (
 	// The path of the proc filesystem.
-	procPath = kingpin.Flag("path.procfs", "procfs mountpoint.").Default(procfs.DefaultMountPoint).String()
-	sysPath  = kingpin.Flag("path.sysfs", "sysfs mountpoint.").Default("/sys").String()
+	procPath   = kingpin.Flag("path.procfs", "procfs mountpoint.").Default(procfs.DefaultMountPoint).String()
+	sysPath    = kingpin.Flag("path.sysfs", "sysfs mountpoint.").Default("/sys").String()
+	rootfsPath = kingpin.Flag("path.rootfs", "rootfs mountpoint.").Default("/").String()
 )
 
 func procFilePath(name string) string {
@@ -33,3 +34,7 @@ func procFilePath(name string) string {
 func sysFilePath(name string) string {
 	return path.Join(*sysPath, name)
 }
+
+func rootfsFilePath(name string) string {
+	return path.Join(*rootfsPath, name)
+}
