@@ -193,7 +193,6 @@ func (c *textFileCollector) Update(ch chan<- prometheus.Metric) error {
 		error = 1.0
 	}
 
-fileLoop:
 	for _, f := range files {
 		if !strings.HasSuffix(f.Name(), ".prom") {
 			continue
@@ -213,14 +212,13 @@ fileLoop:
 			error = 1.0
 			continue
 		}
+		if hasTimestamps(parsedFamilies) {
+			log.Errorf("Textfile %q contains unsupported client-side timestamps, skipping entire file", path)
+			error = 1.0
+			continue
+		}
+
 		for _, mf := range parsedFamilies {
-			for _, m := range mf.Metric {
-				if m.TimestampMs != nil {
-					log.Errorf("Textfile %q contains unsupported client-side timestamps, skipping entire file", path)
-					error = 1.0
-					continue fileLoop
-				}
-			}
 			if mf.Help == nil {
 				help := fmt.Sprintf("Metric read from %s", path)
 				mf.Help = &help
@@ -249,3 +247,15 @@ fileLoop:
 	)
 	return nil
 }
+
+// hasTimestamps returns true when metrics contain unsupported timestamps.
+func hasTimestamps(parsedFamilies map[string]*dto.MetricFamily) bool {
+	for _, mf := range parsedFamilies {
+		for _, m := range mf.Metric {
+			if m.TimestampMs != nil {
+				return true
+			}
+		}
+	}
+	return false
+}
